/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect } from 'react';
//imports material ui components
import { TextField, Button } from '@material-ui/core';
import CircularProgress from '@material-ui/core/CircularProgress';
import SaveIcon from '@material-ui/icons/Save';
import Card from "@material-ui/core/Card";
import CardContent from "@material-ui/core/CardContent";
import Typography from "@material-ui/core/Typography";
import ListRoundedIcon from "@material-ui/icons/ListRounded";
import Grid from "@material-ui/core/Grid";
//imports local files
import SnackBar from '../../../../Components/SnackbarComponent';
//imports services 
import { Edit } from '../../../Services';
import { EditFilter, GetAData } from '../../../Filters'; 
//Routers
import {Link} from 'react-router-dom';

let id;

const EditInstitution = ({ match }) => {
    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete

    const [name, setName] = useState();
    const [description, setDescription] = useState();
    const [adress, setAdress] = useState();
    const [city, setCity] = useState();
    const [country, setCountry] = useState();

    //Used to display circle progress indicator when user click to save
    const [isLoading, setIsLoading] = useState(false);

    const [errorInName, setErrorInName] = useState({
        error: false,
        message: '',
    })

    const [errorInCountry, setErrorInCountry] = useState({
        error: false,
        message: '',
    })

    const [snackInfo, setSnackInfo] = useState({
        message: '',
        icon: '',
        open: false,
        color: '',
    })

    const NameHandler = (e) => {
        if (errorInName.error) {
            setErrorInName({
                error: false,
                message: ''
            })
        }
        setName(e.target.value)
    }
    const DescriptionHandler = (e) => {
        setDescription(e.target.value)
    }
    const AdressHandler = (e) => {
        setAdress(e.target.value)
    }
    const CityHandler = (e) => {
        setCity(e.target.value)
    }
    const CountryHandler = (e) => {
        if (errorInCountry.error) {
            setErrorInCountry({
                error: false,
                message: ''
            })
        }
        setCountry(e.target.value)
    }

    // verify if the given text is empty
    const isEmpty = (text) => {
        return text.length === 0 ? true : false;
    }

    // Handle snack infos
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color
        })
    }

    //Handle submit 
    async function onSubmit() {
        setIsLoading(true)
        if (!isEmpty(name) && !isEmpty(country)) {
            const api = EditFilter('institutions', id)
            const body = {
                "institution": {
                    'name': name,
                    'description': description,
                    'address': adress,
                    'city': city,
                    'country': country,
                }
            }
            console.log(api)
            Edit(api, body).then(res => {
                if (res) {
                    HandleSnack('A instituição foi alterada com sucesso', true, 'success', '#228B22')
                } else {
                    HandleSnack('Ocorreu algum erro', true, 'warning', '#FA8072')
                }
                setIsLoading(false)
            })

        } else {
            HandleSnack('Você precisa preencher algumas informações obrigatórias', true, 'warning', '#FFC125')
            if (isEmpty(name)) {
                setErrorInName({
                    error: true,
                    message: 'Esse campo está vazio'
                })
            }
            if (isEmpty(country)) {
                setErrorInCountry({
                    error: true,
                    message: 'Esse campo está vazio'
                })
            }
            setIsLoading(false)
        }
    }

    // Fields
    const fields = [
        {
            label: 'Nome',
            value: name,
            required: true,
            error: errorInName.error,
            errorMessage: errorInName.message,
            onChange: (event) => NameHandler(event)
        },
        {
            label: 'Descrição',
            value: description,
            required: false,

            onChange: (event) => DescriptionHandler(event)
        },
        {
            label: 'Endereço',
            value: adress,
            required: false,

            onChange: (event) => AdressHandler(event)
        },
        {
            label: 'Cidade',
            value: city,
            required: false,
            onChange: (event) => CityHandler(event)
        },
        {
            label: 'País',
            value: country,
            required: true,
            error: errorInCountry.error,
            errorMessage: errorInCountry.message,
            onChange: (event) => CountryHandler(event)
        }
    ]

    useEffect(() => {
        fetch(GetAData("institutions", match.params.id))
            .then((res) => res.json())
            .then(
                (result) => {
                    setIsLoaded(true);
                    setName(result.name)
                    setDescription(result.description)
                    setAdress(result.adress)
                    setCity(result.city)
                    setCountry(result.country)
                    id = result.id
                },
                (error) => {
                    setIsLoaded(true);
                    setError(error);
                }
            );
    }, []);

    if (error) {
        return <div> Erro... </div>
    } else if (!isLoaded) {
        return <div> Loading... </div>
    } else {
        return (
            <Card>
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() => setSnackInfo({
                        message: '',
                        icon: '',
                        open: false,
                        color: ''
                    })}
                />
                <CardContent>
                    <Grid container xs={12} justify="space-between" alignItems="center" alignContent="center">
                        <Grid item>  
                            <Typography
                                variant='h4'
                                color="inherit"
                                gutterBottom
                            >
                                {name}
                            </Typography>
                        </Grid>

                        <Grid item>
                            <Link style={{textDecoration: 'none'}} to={`/admin/intitutions`}>
                                <Button
                                    startIcon={<ListRoundedIcon />}
                                    color="primary"
                                    variant="outlined"
                                >
                                    Listar
                                </Button>
                            </Link>
                        </Grid>
                    </Grid>

                    <div>
                        <div>
                            <form style={{ display: 'flex', flexDirection: 'column' }}>
                                {fields.map((field, index) => (
                                    <TextField
                                        key={index}
                                        required={field.required}
                                        error={field.error}
                                        helperText={field.error ? field.errorMessage : ''}
                                        style={{ width: '250px', marginBottom: '1em' }}
                                        label={field.label}
                                        value={field.value}
                                        onChange={field.onChange}
                                        type="search"
                                        multiline={true}
                                    />
                                ))}
                            </form>
                        </div>
                        <div>
                            <Button
                                onClick={() => {
                                    onSubmit();
                                }}
                                variant="contained"
                                color="primary"
                                disabled={isLoading}
                                startIcon={isLoading ? null : <SaveIcon />}
                            >
                                {
                                    isLoading ? <CircularProgress size={24} /> : 'Salvar'
                                }
                            </Button>
                        </div>
                    </div>

                </CardContent>
            </Card>

        );
    }
}

export default EditInstitution;