/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect } from "react";
// Imports from local files
import TableData from "../../../Components/Components/Table";
import SnackBar from "../../../../Components/SnackbarComponent";
import AlertDialog from "../../../Components/Components/AlertDialog";
// Imports about icon
import FilterListRoundedIcon from "@material-ui/icons/FilterListRounded";
import AddRoundedIcon from "@material-ui/icons/AddRounded";
import UpdateRoundedIcon from "@material-ui/icons/UpdateRounded";
import TableBody from "@material-ui/core/TableBody";
import TableCell from "@material-ui/core/TableCell";
import TableRow from "@material-ui/core/TableRow";
import IconButton from "@material-ui/core/IconButton";
import CancelRoundedIcon from "@material-ui/icons/CancelRounded";
import VisibilityIcon from "@material-ui/icons/Visibility";
import DeleteIcon from "@material-ui/icons/Delete";
// Import from material-ui
import { withStyles, makeStyles } from "@material-ui/core/styles";
import Paper from "@material-ui/core/Paper";
import Button from "@material-ui/core/Button";
import Grid from "@material-ui/core/Grid";
import { Typography, CircularProgress } from "@material-ui/core";
import Popover from "@material-ui/core/Popover";
import TextField from "@material-ui/core/TextField";
// services
import { Delete, GetFullList } from "../../../Services";
//Filters
import { Url, GetOneOfAllUrl, EditFilter } from "../../../Filters";
//router 
import { Link } from 'react-router-dom';

let currPage = 0;
let transformListToAsc = false

const StyledTableCell = withStyles((theme) => ({
    head: {
        backgroundColor: theme.palette.common.black,
        color: theme.palette.common.white,
    },
    body: {
        fontSize: 14,
    },
}))(TableCell);

const StyledTableRow = withStyles((theme) => ({
    root: {
        "&:nth-of-type(odd)": {
            backgroundColor: theme.palette.action.hover,
        },
    },
}))(TableRow);

const useStyles = makeStyles((theme) => ({
    root: {
        flexGrow: 1,
    },
    paper: {
        padding: theme.spacing(1),
        textAlign: "start",
    },
    button: {
        margin: theme.spacing(1),
        alignSelf: "flex-end",
    },
}));

const Institutions = () => {
    const classes = useStyles();

    const addOndeLenght = [""];

    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
    const [items, setItems] = useState([]); //Necessary to consult the API, data

    const [deleteItem, setDeleteItem] = useState({}); //Delete Item
    const [isLoadingToDelete, setIsLoadingToDelete] = useState(null);
    const [isUpdating, setIsUpdating] = useState(false);
    const [isLoadingMoreItems, setIsLoadingMoreItems] = useState(false);

    const [openAlertDialog, setOpenAlertDialog] = useState(false);

    const [snackInfo, setSnackInfo] = useState({
        message: "",
        icon: "",
        open: false,
        color: "",
    });

    // **************** About the PopOver Menu ****************
    const [anchorEl, setAnchorEl] = React.useState(null);
    const [showCityField, setShowCityField] = useState(false); //show the text field of filter by city
    const [showContryField, setShowCountryField] = useState(false); //show the text field of the filter by country
    const [showDescriptionField, setShowDescriptionField] = useState(false); //show the text field of the filter by description
    const [showStandadSearch, setShowStandarSearchField] = useState(false);

    const [isFilteringByName, setisFilteringByName] = useState(false);
    const [isFilteringByCity, setisFilteringByCity] = useState(false);
    const [isFilteringByCounty, setisFilteringByCountry] = useState(false);
    const [isFilteringByDesc, setisFilteringByDesc] = useState(false);

    const [search, setSeacrh] = useState("");
    const [city, setCity] = useState("");
    const [country, setCountry] = useState("");
    const [description, setDescription] = useState("");

    const CityHandler = () => {
        currPage = 0;
        transformListToAsc = false
        setShowCityField(!showCityField);
        setisFilteringByCity(!isFilteringByCity);
        UpdtateListData(Url("institutions", "", `${currPage}`, "DESC"));
    };

    const CountryHandler = () => {
        currPage = 0;
        transformListToAsc = false
        setShowCountryField(!showContryField);
        UpdtateListData(Url("institutions", "", `${currPage}`, "DESC"));
        setisFilteringByCountry(!isFilteringByCounty);
    };

    const DescHandler = () => {
        currPage = 0;
        transformListToAsc = false
        setShowDescriptionField(!showDescriptionField);
        setisFilteringByDesc(!isFilteringByDesc);
        UpdtateListData(Url("institutions", "", `${currPage}`, "DESC"));
    };

    const StandartHandler = () => {
        currPage = 0;
        transformListToAsc = false
        setShowStandarSearchField(!showStandadSearch);
        setisFilteringByName(!isFilteringByName);
        UpdtateListData(Url("institutions", "", `${currPage}`, "DESC"));
    };

    const handleClick = (event) => {
        setAnchorEl(event.currentTarget);
    };

    const handleClose = () => {
        setAnchorEl(null);
    };

    const open = Boolean(anchorEl);
    const id = open ? "simple-popover" : undefined;

    const OnChangeSearchHandler = (e) => {
        setSeacrh(e.target.value);
        Filter(GetOneOfAllUrl("institutions", `"name" : "${search}"`));
    };

    const onChangeCityHandler = (e) => {
        setCity(e.target.value);
        Filter(GetOneOfAllUrl("institutions", `"city" : "${city}"`));
    };

    const onChangeCountryHandler = (e) => {
        setCountry(e.target.value);
        Filter(GetOneOfAllUrl("institutions", `"country" : "${country}"`));
    };

    const onChangeDescriptionHandler = (e) => {
        setDescription(e.target.value);
        Filter(GetOneOfAllUrl("institutions", `"description" : "${description}"`));
    };
    // **************** About the PopOverMenu ****************

    //Controlls the state of the Alert Dialog
    const HandleStateAlertDialog = (i) => {
        const obj = { ...items[i] };
        setDeleteItem(obj);
        setOpenAlertDialog(!openAlertDialog);
    };

    //Controlls the state and the informations of the snack
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color,
        });
    };

    //Defines which row must show the circular progress
    const HandleStateCircularProgress = (i) => {
        setIsLoadingToDelete(i);
    };

    //Filters the search
    const Filter = (api) => {
        GetFullList(api).then((res) => {
            const arrData = [...res.data];
            if (res.state) {
                transformListToAsc = false
                setItems(arrData.concat(addOndeLenght));
            } else {
                setError(true);
            }
        });
    };

    //This function updates List every time the content of the api changes
    const UpdtateListData = (api) => {
        GetFullList(api).then((res) => {
            const arrData = [...res.data];
            if (res.state) {
                setItems(arrData.concat(addOndeLenght));
                HandleSnack(
                    "A lista de dados foi atualizada",
                    true,
                    "success",
                    "#228B22"
                );
            } else {
                HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
            }
            setIsUpdating(false);
        });
    };

    //Called when user want to delete one institution
    async function DeleteHandler() {
        const id = deleteItem.id;
        HandleStateAlertDialog(null);
        Delete(EditFilter("institutions", id)).then((res) => {
            if (res) {
                HandleSnack(
                    "A instituição foi deletada com sucesso",
                    true,
                    "success",
                    "#228B22"
                );
                currPage = 0;
                transformListToAsc = false
                UpdtateListData(Url("institutions", "", `${currPage}`, "DESC"));
            } else {
                HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
            }
            HandleStateCircularProgress(null);
        });
    }

    const LoadMoreItens = async (api) => {
        setIsLoadingMoreItems(true);
        GetFullList(api).then((res) => {
            if (res.state) {
                const arrData = [...res.data];
                console.log(arrData);
                if (arrData.length === 0) {
                    HandleSnack(
                        "Não há mais dados para serem carregados",
                        true,
                        "warning",
                        "#FFC125"
                    );
                } else {
                    const arrItems = [...items];
                    arrItems.pop(); //Deleting the last position, that was used to display the button of load more items
                    const arrResult = arrItems.concat(arrData);
                    setItems(arrResult.concat(addOndeLenght));
                }
            } else {
                HandleSnack("Erro ao carregar os dados", true, "warning", "#FA8072");
            }
            setIsLoadingMoreItems(false);
        });
    };

    const InvertList = async () => {
        transformListToAsc = !transformListToAsc
        currPage = 0
        if (transformListToAsc) {
            GetFullList(Url('institutions', '', `${currPage}`, 'ASC')).then(res => {
                if (res.state) {
                    const arrData = [...res.data]
                    setItems(arrData.concat(addOndeLenght))
                } else {
                    HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                }
            })
        } else {
            GetFullList(Url('institutions', '', `${currPage}`, 'DESC')).then(res => {
                if (res.state) {
                    const arrData = [...res.data]
                    setItems(arrData.concat(addOndeLenght))
                } else {
                    HandleSnack('institutions ao carregar os dados', true, 'warning', '#FA8072')
                }
            })
        }
    }

    useEffect(() => {
        GetFullList(Url("institutions", "", `${currPage}`, "DESC"))
        .then(
            (result) => {
                if(result.state){
                    setIsLoaded(true);
                    setItems(result.data.concat(addOndeLenght));
                } else setError(true);
            },
        );
            
    }, []);

    if (error) {
        return <div>Error: {error.message}</div>;
    }
    if (!isLoaded) {
        return <div>Loading...</div>;
    } else {
        //Words that defines that column
        const topTable = [
            "ID",
            "NOME",
            "DESCRIÇÃO",
            "CIDADE",
            "PAÍS",
            "VISUALIZAR",
            "DELETAR",
        ];

        //Buttons from PopOverMenu
        const flatButtonsFromPopOverMenu = [
            {
                label: "Pesquisa padrão",
                onClick: StandartHandler,
                color: showStandadSearch ? "primary" : "default",
            },
            {
                label: "Cidade",
                onClick: CityHandler,
                color: showCityField ? "primary" : "default",
            },
            {
                label: "País",
                onClick: CountryHandler,
                color: showContryField ? "primary" : "default",
            },
            {
                label: "Descrição",
                onClick: DescHandler,
                color: showDescriptionField ? "primary" : "default",
            },
        ];

        //Field of the Filter
        const TextFieldOfTheFilter = [
            {
                label: "Pesquisar",
                show: showStandadSearch,
                onChange: (event) => OnChangeSearchHandler(event),
                hide: StandartHandler,
            },
            {
                label: "Cidade",
                show: showCityField,
                onChange: (event) => onChangeCityHandler(event),
                hide: CityHandler,
            },
            {
                label: "País",
                show: showContryField,
                onChange: (event) => onChangeCountryHandler(event),
                hide: CountryHandler,
            },
            {
                label: "Descrição",
                show: showDescriptionField,
                onChange: (event) => onChangeDescriptionHandler(event),
                hide: DescHandler,
            },
        ];

        return (
            <div>
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() =>
                        setSnackInfo({
                            message: "",
                            icon: "",
                            open: false,
                            color: "",
                        })
                    }
                />
                {/************** Start of the header **************/}
                <Paper style={{ padding: "1em" }}>
                    <Grid container spacing={3} direction="row" alignItems="center">
                        <Grid item xs={6}>
                            <Typography className={classes.paper} variant="h4">
                                Lista de Instituições
                            </Typography>
                        </Grid>
                        <Grid
                            item
                            xs={6}
                            alignItems="center"
                            justify="center"
                            direction="row"
                        >
                            <Grid container justify="flex-end" spacing={3}>
                                <Grid item>
                                    <Button
                                        aria-describedby={id}
                                        variant="contained"
                                        color="secondary"
                                        className={classes.button}
                                        onClick={handleClick}
                                        startIcon={
                                            <FilterListRoundedIcon style={{ fill: "white" }} />
                                        }
                                    >
                                        Filtrar
                                    </Button>
                                    <Popover
                                        id={id}
                                        open={open}
                                        anchorEl={anchorEl}
                                        onClose={handleClose}
                                        anchorOrigin={{
                                            vertical: "bottom",
                                            horizontal: "center",
                                        }}
                                        transformOrigin={{
                                            vertical: "top",
                                            horizontal: "center",
                                        }}
                                    >
                                        {flatButtonsFromPopOverMenu.map((flat, index) => (
                                            <Button
                                                key={index}
                                                onClick={flat.onClick}
                                                color={flat.color}
                                            >
                                                {flat.label}
                                            </Button>
                                        ))}
                                    </Popover>
                                </Grid>

                                <Grid item>
                                    <Link style={{textDecoration: 'none'}} to={'/admin/InstitutionCreate'}>
                                        <Button
                                            variant="contained"
                                            color="secondary"
                                            className={classes.button}
                                            startIcon={<AddRoundedIcon style={{ fill: "white" }} />}
                                            onClick={() => {
                                                currPage = 0
                                                transformListToAsc = false
                                            }}
                                        >
                                            Novo
                                        </Button>
                                    </Link>
                                </Grid>

                                <Grid item>
                                    <Button
                                        variant="contained"
                                        color="secondary"
                                        className={classes.button}
                                        startIcon={<UpdateRoundedIcon />}
                                        disabled={isUpdating}
                                        onClick={() => {
                                            currPage = 0;
                                            transformListToAsc = false
                                            setIsUpdating(true);
                                            UpdtateListData(
                                                Url("institutions", "", `${currPage}`, "DESC")
                                            );
                                        }}
                                    >
                                        {isUpdating ? <CircularProgress /> : "Atualizar"}
                                    </Button>
                                </Grid>
                            </Grid>
                        </Grid>
                    </Grid>
                    <Grid item xs={12}>
                        <Grid container justify="space-between" spacing={3}>
                            {TextFieldOfTheFilter.map((field, index) => (
                                <Grid item>
                                    {field.show ? (
                                        <div>
                                            <TextField
                                                id={index}
                                                label={field.label}
                                                type="search"
                                                onChange={field.onChange}
                                            />
                                            <IconButton
                                                size="small"
                                                color="primary"
                                                onClick={field.hide}
                                            >
                                                <CancelRoundedIcon />
                                            </IconButton>
                                        </div>
                                    ) : null}
                                </Grid>
                            ))}
                        </Grid>
                    </Grid>
                </Paper>
                {/************** End of the header **************/}

                <div style={{ height: "2em" }}></div>

                {/************** Start of display data in table **************/}
                <TableData top={topTable} onIconPressed={InvertList}>
                    <TableBody>
                        {items.map((row, index) =>
                            index === items.length - 1 ? (
                                <div style={{ padding: "1em" }}>
                                    {/* Button to load more data */}
                                    <Button
                                        color="primary"
                                        variant="text"
                                        startIcon={<AddRoundedIcon />}
                                        disabled={
                                            isLoadingMoreItems ||
                                            isFilteringByCity ||
                                            isFilteringByCounty ||
                                            isFilteringByDesc ||
                                            isFilteringByName
                                        }
                                        onClick={() => {
                                            currPage++
                                            if (transformListToAsc) {
                                                LoadMoreItens(Url('institutions', '', `${currPage}`, 'ASC'))
                                            } else {
                                                LoadMoreItens(Url('institutions', '', `${currPage}`, 'DESC'))
                                            }
                                        }}
                                    >
                                        {isLoadingMoreItems ? (
                                            <CircularProgress />
                                        ) : (
                                                "Carregar mais itens"
                                            )}
                                    </Button>
                                </div>
                            ) : (
                                    <StyledTableRow key={index}>
                                        <StyledTableCell component="th" scope="row">
                                            {row.id}
                                        </StyledTableCell>
                                        <StyledTableCell align="right">{row.name}</StyledTableCell>
                                        <StyledTableCell align="right">
                                            {row.description}
                                        </StyledTableCell>
                                        <StyledTableCell align="right">{row.city}</StyledTableCell>
                                        <StyledTableCell align="right">
                                            {row.country}
                                        </StyledTableCell>
                                        <StyledTableCell align="right">
                                            <Link to={`/admin/institution/${row.id}`}>
                                                <IconButton onClick={() => {
                                                    currPage = 0
                                                    transformListToAsc = false
                                                }}>
                                                    <VisibilityIcon style={{ fill: "#00bcd4" }} />
                                                </IconButton>
                                            </Link>
                                        </StyledTableCell>
                                        <StyledTableCell align="right">
                                            {isLoadingToDelete === index ? (
                                                <CircularProgress size={24} color="primary" />
                                            ) : (
                                                    <IconButton
                                                        onClick={() => {
                                                            HandleStateAlertDialog(index);
                                                            HandleStateCircularProgress(index);
                                                        }}
                                                    >
                                                        <DeleteIcon style={{ fill: "#FF0000" }} />
                                                    </IconButton>
                                                )}
                                        </StyledTableCell>
                                    </StyledTableRow>
                                )
                        )}
                    </TableBody>
                </TableData>
                {/************** End of display data in table **************/}

                {/* This alert will be displayed if the user click to delete an institution */}
                <AlertDialog
                    open={openAlertDialog}
                    OnDelete={DeleteHandler}
                    deleteItem={deleteItem}
                    HandleClose={() => {
                        setOpenAlertDialog(false);
                        HandleStateCircularProgress(null);
                    }}
                />
            </div>
        )
    }
};

export default Institutions;
